<?php

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2012 Thomas Dudzak <thomas.dudzak@gmx.net>
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package td_rssfeeds
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class Tx_TdRssfeeds_Controller_FeedController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {
	
	var $xml;
	
	var $data 			= array();
	var $currentTag 	= '';
	var $imageExists 	= FALSE;
	var $inputExists 	= FALSE;
	var $channelExists 	= FALSE;
	var $itemExists 	= FALSE; 
	var $itemCount 		= 0;
	var $caseFolding 	= TRUE;
	var $encoding		= ''; 

	public function findRssEntries($file, $encoding, $limit="5") {
		$this->encoding = $encoding; 
		$this->parseXml($file); 
		$this->setMaximalItemsToLimit($limit); 
		
		return $this->data;
	}
	
	private function parseXml($file) {
		$this->xml = xml_parser_create();
		xml_set_object($this->xml, $this);
		xml_parser_set_option($this->xml, XML_OPTION_CASE_FOLDING, $this->caseFolding);
		xml_set_element_handler($this->xml, 'startElementParser', 'endElementParser');
		xml_set_character_data_handler($this->xml, 'setCharacterDataHandler');
		
		if (!($fc = @fopen($file, "r")))
			//echo "cant open xml"; //$this->error(sprintf($this->pi_getLL('cant_open_xml'), $file));
			if (TYPO3_DLOG) \TYPO3\CMS\Core\Utility\GeneralUtility::devLog('Can\'t open XML-Source', 'td_rssfeeds');

		if (empty($this->error)) {
			while ($data = @fread($fc, 4096)) {
				if (!(xml_parse($this->xml, $data)))
					//echo 'XML-Output: '.xml_error_string(xml_get_error_code($this->xml));  //$this->error('XML-Output: '.xml_error_string(xml_get_error_code($this->xml)));
					if (TYPO3_DLOG) \TYPO3\CMS\Core\Utility\GeneralUtility::devLog('XML-Output: '.xml_error_string(xml_get_error_code($this->xml)), 'td_rssfeeds');
			}
		}
		xml_parser_free($this->xml);
		
		return;
	}
	
	function startElementParser($xml, $tag, $attribute) {
		if (strtoupper(substr($tag, 0, 3)) == 'rdf') {
			$tag = substr($tag, 4, strlen($tag));
		}

		$this->currentTag = $this->stringToLowercase($tag);

		if ($this->stringToLowercase($tag) == 'channel') {
			$this->channelExists= true;
		}

		if ($this->stringToLowercase($tag) == 'image') {
			$this->imageExists = true;
		}

		if ($this->stringToLowercase($tag) == 'textinput') {
			$this->inputExists = true;
		}

		if ($this->stringToLowercase($tag) == 'item') {
			$this->itemExists = true;
			$this->itemCount++;
		}
	}

	function endElementParser($xml, $tag) {
		if ($this->stringToLowercase($tag) == 'channel') {
			$this->channelExists = false;
		}

		if ($this->stringToLowercase($tag) == 'image') {
			$this->imageExists = false;
		}

		if ($this->stringToLowercase($tag) == 'textinput') {
			$this->inputExists = false;
		}

		if ($this->stringToLowercase($tag) == 'item') {
			$this->itemExists = false;
		}
	}
	
	function setCharacterDataHandler($xml, $data) {
 		if (!empty($this->encoding)) {
			$data = mb_convert_encoding($data,'UTF-8',$this->encoding);
		}
		
		if ($this->channelExists == TRUE && $this->itemExists == FALSE && $this->imageExists == FALSE && $this->inputExists == FALSE) {
			if ($this->currentTag != 'channel') {
				if (!isset($this->data['channel'][$this->currentTag])) {
					$this->data['channel'][$this->currentTag] = '';
				}
				$this->data['channel'][$this->currentTag] .= $data;
			}
		}

		if ($this->imageExists == TRUE) {
			if ($this->currentTag != 'image') {
				if (!isset($this->data['image'][$this->currentTag])) {
					$this->data['image'][$this->currentTag] = '';
				}
				$this->data['image'][$this->currentTag] .= $data;
			}
		}

		if ($this->inputExists == TRUE) {
			if ($this->currentTag != 'textinput') {
				if (!isset($this->data['textinput'][$this->currentTag])) {
					$this->data['textinput'][$this->currentTag] = '';
				}
				$this->data['textinput'][$this->currentTag] .= $data;
			}
		}

		if ($this->itemExists == TRUE) {
			if ($this->currentTag != 'item') {
				if (!isset($this->data['item'][$this->itemCount][$this->currentTag])) {
					$this->data['item'][$this->itemCount][$this->currentTag] = '';
				}
				$this->data['item'][$this->itemCount][$this->currentTag] .= $data;
			}
		}
	}
	
	function stringToLowercase($tag) {
		if ($this->caseFolding)
			return strtolower($tag);
		else
			return $tag;
	}
	
	function setMaximalItemsToLimit($limit) {
		$items= array(); 
		$i=1; 
		
		if ($this->itemCount >= $limit) {
			foreach($this->data['item'] as $key => $value) {
				if ($i++ > $limit) {
					break; 
				}
				$items[$key] = $value; 
			}
			
			$this->data['item'] = $items; 
		}
		return; 
	}
	

	/**
	 * feedRepository
	 *
	 * @var Tx_TdRssfeeds_Domain_Repository_FeedRepository
	 */
	protected $feedRepository;

	/**
	 * injectFeedRepository
	 *
	 * @param Tx_TdRssfeeds_Domain_Repository_FeedRepository $feedRepository
	 * @return void
	 */
	public function injectFeedRepository(Tx_TdRssfeeds_Domain_Repository_FeedRepository $feedRepository) {
		$this->feedRepository = $feedRepository;
	}

	/**
	 * action list
	 *
	 * @return void
	 */
	public function listAction() {
		$demand = $this->overwriteSettingsWithFlexformValues($this->settings);
		$feeds = $this->findRssEntries($demand['url'], $this->getEncodingSettings($demand['encodeCharset']),$demand['maxItems']);
		$feeds['image']['url'] = $demand['channelImage'] == true ? $this->setImageUrlToLocale($feeds['image']['url']) : '';
		$this->view->assignMultiple(array(
			'feeds' => $feeds,
			'demand' => $demand
		));
	}
	
	/**
	 * action listcached
	 *
	 * @return void
	 */
	public function listCachedAction() {
		$demand = $this->overwriteSettingsWithFlexformValues($this->settings);
		$feeds = $this->findRssEntries($demand['url'], $this->getEncodingSettings($demand['encodeCharset']),$demand['maxItems']);
		$feeds['image']['url'] = $demand['channelImage'] == true ? $this->setImageUrlToLocale($feeds['image']['url']) : '';
		$this->view->assignMultiple(array(
			'feeds' => $feeds,
			'demand' => $demand
		));
	}
	
	
	public function setImageUrlToLocale($image='') {
		if (!empty($image)) {
			$allowed = array('jpeg', 'jpg', 'png', 'gif'); 
			$targetType = trim(substr(strrchr($image, '.'),1)); 
			$targetPath = 'uploads/pics/';
			$targetName = 'tdrf_'.md5($image).'.'.$targetType; 
	
			if (in_array($targetType, $allowed)) {
				if(!file_exists($targetPath.$targetName)) {
					file_put_contents($targetPath.$targetName,\TYPO3\CMS\Core\Utility\GeneralUtility::getUrl(trim($image)));
				} 
				
				return $targetPath.$targetName; 
			} else {
				return ''; 
			}
		}
		return ''; 
	}
	
	public function overwriteSettingsWithFlexformValues($demand) {
		foreach ($demand['ff'] as $key=>$value) {
			if (!empty($value)) {
					$demand[$key] = $value; 
			}
		}
		
		return $demand; 
	}
	
	public function getEncodingSettings($encodingCharset) {
		switch($encodingCharset){
			case 'NONE':
			$charset = '';
			break;
				
			case 'UTF8':
			$charset = 'UTF-8';				
			break;
				
			case 'SJIS':
			$charset = 'EUC-JP';
			break;	

			case 'EUCJP':
			$charset ='SJIS';
			break;

			case 'ISO88591':
			$charset = 'iso-8859-1';
			break;
				
			case 'ISO88592':
			$charset = 'iso-8859-2';
			break;				

			case 'ISO88593':
			$charset = 'iso-8859-3';
			break;
			
			case 'ISO88594':
			$charset = 'iso-8859-4';
			break;
			
			case 'ISO88595':
			$charset = 'iso-8859-5';
			break;	
			
			case 'ISO88596':
			$charset = 'iso-8859-6';
			break;	
			
			case 'ISO88597':
			$charset = 'iso-8859-7';
			break;	
			
			case 'ISO88598':
			$charset = 'iso-8859-8';
			break;																	
			
			case 'AUTO':
			$charset = 'auto';				
			break;							
			
			default:
			$charset = '';
			break;
		}
	
		return $charset; 
	}
	
}
?>